/*
 * DemosceneMusic - Artist.java - Copyright © 2012 David Roden
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.pterodactylus.demoscenemusic.data;

import java.util.Collection;

/**
 * Data interface for artists.
 *
 * @author <a href="mailto:bombe@pterodactylus.net">David ‘Bombe’ Roden</a>
 */
public interface Artist extends Base {

	/**
	 * Returns the name of this artist.
	 *
	 * @return The name of this artist
	 */
	public String getName();

	/**
	 * Sets the name of this artist.
	 *
	 * @param name
	 *            The new name of this artist
	 * @return This artist
	 */
	public Artist setName(String name);

	/**
	 * Returns the groups this artist belongs to.
	 *
	 * @return The groups this artist belongs to
	 */
	public Collection<Group> getGroups();

	/**
	 * Sets the groups this artist belongs to.
	 *
	 * @param groups
	 *            The groups this artist belongs to
	 * @return This artist
	 */
	public Artist setGroups(Collection<Group> groups);

	/**
	 * Returns the tracks created by this artist.
	 *
	 * @return The tracks created by this artist
	 */
	public Collection<Track> getTracks();

	/**
	 * Sets the tracks created by this artist.
	 *
	 * @param tracks
	 *            The tracks created by this artist
	 * @return This artist
	 */
	public Artist setTracks(Collection<Track> tracks);

}
