/*
 * DemosceneMusic - AbstractParty.java - Copyright © 2012 David Roden
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.pterodactylus.demoscenemusic.data;

import java.util.Collection;
import java.util.Comparator;

import net.pterodactylus.util.number.Numbers;

/**
 * Default implementation of a party data container.
 *
 * @author <a href="mailto:bombe@pterodactylus.net">David ‘Bombe’ Roden</a>
 */
public class DefaultParty extends DefaultBase implements Party {

	/** Comparator that sorts parties by name and descending year. */
	public static final Comparator<Party> NAME_YEAR_COMPARATOR = new Comparator<Party>() {

		@Override
		public int compare(Party leftParty, Party rightParty) {
			int diff = leftParty.getName().compareToIgnoreCase(rightParty.getName());
			if (diff != 0) {
				return diff;
			}
			int leftYear = Numbers.safeParseInteger(leftParty.getProperties().get("party/year"), 0);
			int rightYear = Numbers.safeParseInteger(rightParty.getProperties().get("party/year"), 0);
			/* show newest parties first. */
			return rightYear - leftYear;
		}

	};

	/**
	 * Creates a new party data container.
	 *
	 * @param id
	 *            The ID of the party
	 */
	public DefaultParty(String id) {
		super(id);
	}

	//
	// PARTY METHODS
	//

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String getName() {
		return getValue("name", String.class).get();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public DefaultParty setName(String name) {
		getValue("name", String.class).set(name);
		return this;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	@SuppressWarnings("unchecked")
	public Collection<Track> getReleases() {
		return getValue("releases", Collection.class).get();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public Party setReleases(Collection<Track> tracks) {
		getValue("releases", Collection.class).set(tracks);
		return this;
	}

}
